@@ -21,8 +21,10 @@
 
 #include "atk.h"
 
-static void            atk_relation_set_class_init       (AtkRelationSetClass  *klass);
-static void            atk_relation_set_finalize         (GObject              *object);
+static gpointer parent_class = NULL;
+
+static void atk_relation_set_class_init (AtkRelationSetClass  *klass);
+static void atk_relation_set_finalize   (GObject              *object);
 
 GType
 atk_relation_set_get_type (void)
@@ -53,6 +55,8 @@ atk_relation_set_class_init (AtkRelationSetClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  parent_class = g_type_class_peek_parent (klass);
+
   gobject_class->finalize = atk_relation_set_finalize;
 }
 
@@ -169,7 +173,7 @@ atk_relation_set_add (AtkRelationSet *set,
 gint
 atk_relation_set_get_n_relations (AtkRelationSet *set)
 {
-  g_return_val_if_fail (ATK_IS_RELATION_SET (set), FALSE);
+  g_return_val_if_fail (ATK_IS_RELATION_SET (set), 0);
 
   if (set->relations == NULL)
     return 0;
@@ -193,7 +197,7 @@ atk_relation_set_get_relation (AtkRelationSet *set,
   GPtrArray *array_item;
   AtkRelation* item;
 
-  g_return_val_if_fail (ATK_IS_RELATION_SET (set), FALSE);
+  g_return_val_if_fail (ATK_IS_RELATION_SET (set), NULL);
   g_return_val_if_fail (i >= 0, NULL);
 
   array_item = set->relations;
@@ -223,7 +227,7 @@ atk_relation_set_get_relation_by_type (AtkRelationSet  *set,
   AtkRelation *item;
   gint i;
 
-  g_return_val_if_fail (ATK_IS_RELATION_SET (set), FALSE);
+  g_return_val_if_fail (ATK_IS_RELATION_SET (set), NULL);
 
   array_item = set->relations;
   if (array_item == NULL)
@@ -257,4 +261,43 @@ atk_relation_set_finalize (GObject *object)
     }
     g_ptr_array_free (array, TRUE);
   }
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
+
+/**
+ * atk_relation_set_add_relation_by_type:
+ * @set: an #AtkRelationSet
+ * @relationship: an #AtkRelationType
+ * @target: an #AtkObject
+ *
+ * Add a new relation of the specified type with the specified target to 
+ * the current relation set if the relation set does not contain a relation
+ * of that type. If it is does contain a relation of that typea the target
+ * is added to the relation.
+ **/
+void
+atk_relation_set_add_relation_by_type (AtkRelationSet  *set,
+                                       AtkRelationType relationship,
+                                       AtkObject       *target)
+{
+  AtkRelation *relation;
+
+  g_return_if_fail (ATK_IS_RELATION_SET (set));
+  g_return_if_fail (ATK_IS_OBJECT (target));
+
+  relation = atk_relation_set_get_relation_by_type (set,
+                                                    relationship);
+  if (relation)
+    {
+      atk_relation_add_target (relation, target);
+    } 
+  else 
+    {
+      /* the relation hasn't been created yet ... */
+      relation = atk_relation_new (&target, 1, relationship);
+      atk_relation_set_add (set, relation);
+      g_object_unref(relation);
+    }
+}
+
